@@ -3,13 +3,13 @@
 
    Model File:  perc.model
 
-   Date:  Sun Oct 25 22:46:16 2015
+   Date:  Mon May  9 16:22:27 2016
 
-   Created by:  "mod v5.5.0"
+   Created by:  "mod v5.6.5"
     -- a model preprocessor by Don Maszle
    ___________________________________________________
 
-   Copyright (c) 1993-2013 Free Software Foundation, Inc.
+   Copyright (c) 1993-2015 Free Software Foundation, Inc.
 
    Model calculations for compartmental model:
 
@@ -147,7 +147,7 @@
 /* For export. Keep track of who we are. */
 char szModelDescFilename[] = "perc.model";
 char szModelSourceFilename[] = __FILE__;
-char szModelGenAndVersion[] = "mod v5.5.0";
+char szModelGenAndVersion[] = "mod v5.6.5";
 
 /* Externs */
 extern BOOL vbModelReinitd;
@@ -203,6 +203,8 @@ double Flow_tot;
 double Flow_alv;
 double Vmax;
 
+BOOL bDelays = 0;
+
 
 /*----- Global Variable Map */
 
