@@ -320,7 +320,7 @@ int FillBuffer (PINPUTBUF pibIn, long lBuffer_size)
 
     if ((iOffset = fread (pibIn->pbufOrg, 1, lBuffer_size, pibIn->pfileIn))) {
       iReturn = (int) iOffset;
-      PreventLexSplit (pibIn, iOffset);
+      // PreventLexSplit (pibIn, iOffset);
       pibIn->pbufCur = pibIn->pbufOrg;
     } /* if */
 
