@@ -67,7 +67,7 @@ KM vrgkmKeywordMap[] = {
   {"SBMLModels",  KM_SBMLMODELS,  CN_GLOBAL | CN_TEMPLATE_DEFINED},
   {"PKTemplate",  KM_PKTEMPLATE,  CN_GLOBAL},
   
-  {"End", KM_END, CN_GLOBAL | CN_TEMPLATE_DEFINED}, /* Optional End statmt */
+  {"End", KM_END, CN_GLOBAL | CN_TEMPLATE_DEFINED}, /* Mandatory End statmt */
 
   /* Variables names valid in all CN_ */
   {"", 0, CN_ALL} /* This marks the end of map too */
@@ -304,6 +304,11 @@ void ProcessWord (PINPUTBUF pibIn, PSTR szLex, PSTR szEqn)
   int iKWCode, fContext;
   PSTRLEX szPunct;
   PINPUTINFO pinfo;
+
+  static BOOL bCalcOutputsDefined = FALSE;
+  static BOOL bDynamicsDefined    = FALSE;
+  static BOOL bInitializeDefined  = FALSE;
+  static BOOL bJacobianDefined    = FALSE;
   
   if (!pibIn || !szLex || !szLex[0] || !szEqn)
     return;
@@ -340,11 +345,9 @@ void ProcessWord (PINPUTBUF pibIn, PSTR szLex, PSTR szEqn)
         break;
       
       case KM_CALCOUTPUTS:
-      case KM_JACOB:
-      case KM_SCALE:
-      case KM_EVENTS:
-      case KM_ROOTS:
-      case KM_DYNAMICS:
+        if (bCalcOutputsDefined)
+          ReportError (pibIn, RE_DUPSECT | RE_FATAL, "CalcOutputs", NULL);
+        bCalcOutputsDefined = TRUE;
         if (!GetPunct (pibIn, szPunct, CH_LBRACE)) {
           szPunct[1] = CH_LBRACE;
           ReportError (pibIn, RE_EXPECTED | RE_FATAL, szPunct,
@@ -353,7 +356,58 @@ void ProcessWord (PINPUTBUF pibIn, PSTR szLex, PSTR szEqn)
         else
           pinfo->wContext = KM_TO_CN(iKWCode);
         break;
-      
+
+      case KM_JACOB:
+        if (bJacobianDefined)
+          ReportError (pibIn, RE_DUPSECT | RE_FATAL, "Jacobian", NULL);
+        bJacobianDefined = TRUE;
+        if (!GetPunct (pibIn, szPunct, CH_LBRACE)) {
+          szPunct[1] = CH_LBRACE;
+          ReportError (pibIn, RE_EXPECTED | RE_FATAL, szPunct,
+                       "* Section must be delimited by curly braces.");
+        }
+        else
+          pinfo->wContext = KM_TO_CN(iKWCode);
+        break;
+
+      case KM_SCALE:
+        if (bInitializeDefined)
+          ReportError (pibIn, RE_DUPSECT | RE_FATAL, "Initialize", NULL);
+        bInitializeDefined = TRUE;
+        if (!GetPunct (pibIn, szPunct, CH_LBRACE)) {
+          szPunct[1] = CH_LBRACE;
+          ReportError (pibIn, RE_EXPECTED | RE_FATAL, szPunct,
+                       "* Section must be delimited by curly braces.");
+        }
+        else
+          pinfo->wContext = KM_TO_CN(iKWCode);
+        break;
+
+      case KM_EVENTS:
+      case KM_ROOTS:
+        if (!GetPunct (pibIn, szPunct, CH_LBRACE)) {
+          szPunct[1] = CH_LBRACE;
+          ReportError (pibIn, RE_EXPECTED | RE_FATAL, szPunct,
+                       "* Section must be delimited by curly braces.");
+        }
+        else
+          pinfo->wContext = KM_TO_CN(iKWCode);
+        break;
+
+      case KM_DYNAMICS:
+        if (bDynamicsDefined)
+          ReportError (pibIn, RE_DUPSECT | RE_FATAL, "Dynamics", NULL);
+        bDynamicsDefined = TRUE;
+
+        if (!GetPunct (pibIn, szPunct, CH_LBRACE)) {
+          szPunct[1] = CH_LBRACE;
+          ReportError (pibIn, RE_EXPECTED | RE_FATAL, szPunct,
+                       "* Section must be delimited by curly braces.");
+        }
+        else
+          pinfo->wContext = KM_TO_CN(iKWCode);
+        break;
+
       case KM_DXDT: /* State equation definition */    
         ProcessDTStatement (pibIn, szLex, szEqn, iKWCode);
         break;
@@ -401,6 +455,40 @@ void ProcessWord (PINPUTBUF pibIn, PSTR szLex, PSTR szEqn)
 } /* ProcessWord */
 
 
+/* ----------------------------------------------------------------------------
+   FindEnd
+   
+   Returns 1 if the keyword "End" is found at the beginning of line, 
+   eventually preceeded by white space, and zero otherwise.
+*/
+
+int FindEnd (PBUF pBuf, long N)
+{
+  char *c, *end;
+
+  c = pBuf;
+  end = pBuf + N;
+  while (c < end) {
+    // printf ("%c", *c);
+    if (*c == CH_EOLN) { // eat up leading white space
+      c++;
+      while ((c < end) && (isspace(*c))) //((*c == ' ') || (*c == '\t')))
+        c++;
+      if (c < end) {
+        // printf ("|%c", *c);
+	if (((c+2) < end) && (*c == 'E') && (*(c+1) == 'n') && (*(c+2) == 'd'))
+	  return (1);
+      }
+    }
+    c++;
+  }
+
+  // not found
+  return(0);
+
+} /* FindEnd */
+
+
 /* ----------------------------------------------------------------------------
    ReadModel
    
@@ -422,6 +510,10 @@ void ReadModel (PINPUTINFO pinfo, PINPUTINFO ptempinfo, PSTR szFileIn)
   ibIn.pInfo = (PVOID) pinfo;
   ibIn.pTempInfo = (PVOID) ptempinfo;
 
+  /* immediately check whether a valid End is found */
+  if (FindEnd(ibIn.pbufOrg, ibIn.lBufSize) == 0)
+    ReportError (NULL, RE_NOEND | RE_FATAL, szFileIn, NULL);
+
   do { /* State machine for parsing syntax */
     NextLex (&ibIn, szLex, &iLexType);
     switch (iLexType) {
