@@ -75,11 +75,12 @@ typedef struct tagINPUTFUNCTIONMAP {
 
 IFM vrgifmMap[] = { /* Input function map */
 
-  {"PerDose",   IFN_PERDOSE},
-  {"PerExp",    IFN_PEREXP},
-  {"NDoses",    IFN_NDOSES},
-  {"Spikes",    IFN_SPIKES},
-  {"Events",    IFN_EVENTS},
+  {"PerDose",    IFN_PERDOSE},
+  {"PerExp",     IFN_PEREXP},
+  {"PerTransit", IFN_PERTRANS},
+  {"NDoses",     IFN_NDOSES},
+  {"Spikes",     IFN_SPIKES},
+  {"Events",     IFN_EVENTS},
 
   {"", IFN_NULL} /* End flag */
 
@@ -184,16 +185,16 @@ BOOL DefDepParm (PSTR szLex, PDOUBLE pdValue, HANDLE *phvar)
      initialization.
 */
 
-BOOL GetInputArgs (PINPUTBUF pibIn, PIFN pifn)
+BOOL GetInputArgs (PINPUTBUF pibIn, PIFN pifn, int nArgs)
 {
-  PSTRLEX rgszLex[4];
-  int rgiTypes[4], i;
+  PSTRLEX rgszLex[nArgs];
+  int rgiTypes[nArgs], i;
   BOOL bReturn = FALSE;
 
-  for (i = 0; i < 4; i++)
+  for (i = 0; i < nArgs; i++)
     rgiTypes[i] = LX_INTEGER | LX_FLOAT | LX_IDENTIFIER;
 
-  if (GetFuncArgs (pibIn, 4, rgiTypes, rgszLex[0])) {
+  if (GetFuncArgs (pibIn, nArgs, rgiTypes, rgszLex[0])) {
 
     /* Try to get each parm to show all errors */
 
@@ -202,11 +203,14 @@ BOOL GetInputArgs (PINPUTBUF pibIn, PIFN pifn)
     bReturn &= DefDepParm (rgszLex[1], &pifn->dTper, &pifn->hTper);
     bReturn &= DefDepParm (rgszLex[2], &pifn->dT0, &pifn->hT0);
 
-    if (pifn->iType == IFN_PEREXP)
+    if ((pifn->iType == IFN_PEREXP) || (pifn->iType == IFN_PERTRANS))
       bReturn &= DefDepParm (rgszLex[3], &pifn->dDecay, &pifn->hDecay);
     else
       bReturn &= DefDepParm (rgszLex[3], &pifn->dTexp, &pifn->hTexp);
 
+    if ((pifn->iType == IFN_PERTRANS) && (nArgs == 5))
+      bReturn &= DefDepParm (rgszLex[4], &pifn->dNcpt, &pifn->hNcpt);
+
     if (!bReturn)
       ReportError (pibIn, RE_EXPECTED, "input-spec", NULL);
   } /* if */
@@ -613,7 +617,11 @@ BOOL GetInputFn (PINPUTBUF pibIn, PSTR sz, PIFN pifn)
 
       case IFN_PERDOSE:
       case IFN_PEREXP:
-        bReturn = GetInputArgs (pibDum, pifn);
+        bReturn = GetInputArgs (pibDum, pifn, 4); /* 4 arguments */
+        break;
+
+      case IFN_PERTRANS:
+        bReturn = GetInputArgs (pibDum, pifn, 5); /* 5 arguments */
         break;
 
       } /* switch */
