@@ -52,6 +52,7 @@
 #define IFN_NDOSES   4
 #define IFN_SPIKES   5
 #define IFN_EVENTS   6
+#define IFN_PERTRANS 7
 
 /* ----------------------------------------------------------------------------
    Typedefs  */
@@ -75,19 +76,21 @@
 
      IFN_PERDOSE  Periodic dose lasting dTexp
      IFN_PEREXP   Periodic exponential with decay constant dDecay
+     IFN_PERTRANS Periodic transit (analytical multicompartment) Savic model
+                  Savic et al., J Pharmacokinet Pharmacodyn. 2007, 34:711.
 
    -- Multiple pulse functions:
 
      IFN_NDOSES   nDoses of rgMags[] starting at rgT0s[]
      IFN_SPIKES   nDoses spikes of rgMags[] at time rgT0s[]
-     IFN_EVENT    nDoses discontinuities in a state variable
+     IFN_EVENTS   nDoses discontinuities in a state variable
 */
 
 typedef struct tagIFN {
   /* Bookkeeping */
 
-  int iType;               /* One of the IFN_ types */
-  BOOL bOn;                /* TRUE if exposure is On */
+  int    iType;            /* One of the IFN_ types */
+  BOOL   bOn;              /* TRUE if exposure is On */
   double dTStartPeriod;    /* Start of current period */
   double dVal;             /* Current value: CalcInputs updates */
 
@@ -101,9 +104,14 @@ typedef struct tagIFN {
   /* For exponential inputs, the exponential decay rate
      Exposure lasts for N_TAU_EXPOSE Tau periods. (tau=1/Decay)
      After this, input is considered to be neglible. */
-
   double dDecay;
 
+  /* For Savic's transit model inputs 
+     dMag * ((dDecay * t)^n) * exp(-dDecay * t) / n!
+     with n! = SQRT2PI * (n^(n+0.5)) *exp(-n)
+     So we need n, the number of virtual transit compartments (as a double) */
+  double dNcpt; 
+
   /* Dependencies for the periodic parms */
 
   HANDLE hMag;             /* Handle to magnitude */
@@ -111,6 +119,7 @@ typedef struct tagIFN {
   HANDLE hT0;              /* Handle to starting time */
   HANDLE hTexp;            /* Handle to exposure time */
   HANDLE hDecay;           /* Handle to exponential decay rate */
+  HANDLE hNcpt;            /* Handle to the number of virtual compartments */
 
   /* Multiple dose inputs */
 
@@ -138,7 +147,7 @@ typedef struct tagIFN {
 BOOL DefDepParm (PSTR szLex, PDOUBLE pdValue, HANDLE *phvar);
 
 int  GetFnType (PSTR szName);
-BOOL GetInputArgs (PINPUTBUF pibIn, PIFN pifn);
+BOOL GetInputArgs (PINPUTBUF pibIn, PIFN pifn, int n);
 BOOL GetInputFn (PINPUTBUF pibIn, PSTR sz, PIFN pifn);
 BOOL GetNDoses (PINPUTBUF pibIn, PSTR szLex, PIFN pifn);
 BOOL GetNNumbers (PINPUTBUF pibIn, PSTR szLex, int nNumbers, PDOUBLE rgd);
