@@ -29,13 +29,14 @@
    -- SCCS  ---------
 
    This file calls at least one GNU Scientific Library function, if the
-   symbol HAVE_LIBGSL is defined in config.h (or in the makefile). 
+   symbol HAVE_LIBGSL is defined in config.h (or in the makefile).
    Otherwise, the corresponding features are disabled (and the program
    exits with an error message).
 */
 
 #include <stdio.h>
 #include <string.h>
+#include <inttypes.h>
 
 #include "mh.h"
 #include "lsodes.h"
@@ -47,6 +48,58 @@
 #endif
 
 
+/* Function -------------------------------------------------------------------
+   AnnounceMarkov
+
+   Print out the type of simulations to be performed.
+*/
+
+void AnnounceMarkov (int nSimTypeFlag, long nIter)
+{
+
+  switch (nSimTypeFlag) {
+
+  case 0:
+    printf ("\nDoing %ld Metropolis within Gibbs simulation",
+            nIter);
+    printf ((nIter != 1 ? "s\n" : "\n"));
+    break;
+
+  case 1:
+    printf ("\nPrinting data and predictions for the last line of the "
+            "restart file\n");
+    break;
+
+  case 2:
+    printf ("\nDoing %ld Metropolis simulation",
+            nIter);
+    printf ((nIter != 1 ? "s\n" : "\n"));
+    break;
+
+  case 3:
+    printf ("\nDoing %ld Metropolis within Gibbs posterior "
+            "tempered simulation",
+            nIter);
+    printf ((nIter != 1 ? "s\n" : "\n"));
+
+    break;
+
+  case 4:
+    printf ("\nDoing %ld Metropolis within Gibbs likelihood "
+            "tempered simulation",
+            nIter);
+    printf ((nIter != 1 ? "s\n" : "\n"));
+
+    break;
+
+  case 5:
+    printf("\nDoing Stochastic optimization\n");
+    break;
+  }
+
+} /* AnnounceMarkov */
+
+
 /* Function -------------------------------------------------------------------
    CalculateTotals
 
@@ -71,7 +124,7 @@ void CalculateTotals (PLEVEL plevel, char **args)
 /* Function -------------------------------------------------------------------
    CheckForFixed
 
-   It is possible for an MC var to be fixed by a subsequent `=' statement in 
+   It is possible for an MC var to be fixed by a subsequent `=' statement in
    the level just below the level at which it is declared in the input file;
    This routine marks such vars as "Fixed" and set their dVal to the
    prescribed number.
@@ -171,7 +224,7 @@ void CloneLikes (PLEVEL plevel, char **args)
 
   for (i = 0; i < plevel->iInstances; i++) {
     pLower = plevel->pLevels[i];
-    /* The number of likelihoods is overestimated by this, but that will do 
+    /* The number of likelihoods is overestimated by this, but that will do
        for now: */
     pLower->nLikes = nLikes = plevel->nLikes + ListLength(plevel->plistLikes);
 
@@ -372,31 +425,38 @@ void ConvertLists(PLEVEL plevel, char **args)
 /* Function -------------------------------------------------------------------
    SetdInvTemperatures
 
-   set default values for inverse temperatures of tempered MCMC algorithm.
+   Set default values for inverse temperatures of tempered MCMC algorithms, if
+   the user has not specified values.
 */
 
 void SetInvTemperatures (PGIBBSDATA pgd) {
 
-  int i;
+  /* inverse temperatures not set by the user: use the default */
+  if (((pgd->nSimTypeFlag == 3) || (pgd->nSimTypeFlag == 4)) &&
+      (pgd->nInvTemperatures == 0)) {
 
-  pgd->nInvTemperatures = NTEMP;
+    int i;
 
-  /* allocate inverse temperature array */
-  if (!(pgd->rgInvTemperatures = InitdVector (NTEMP)))
-    ReportError (NULL, RE_OUTOFMEM | RE_FATAL, "SetInvTemperatures", NULL);
+    pgd->nInvTemperatures = NTEMP;
 
-  /* allocate working arrays */
-  if (!(pgd->rgdlnPi = InitdVector (NTEMP)) || 
-      !(pgd->rglTemp = InitlVector (NTEMP)))
-    ReportError (NULL, RE_OUTOFMEM | RE_FATAL, "SetInvTemperatures", NULL);
+    /* allocate inverse temperature array */
+    if (!(pgd->rgInvTemperatures = InitdVector (NTEMP)))
+      ReportError (NULL, RE_OUTOFMEM | RE_FATAL, "SetInvTemperatures", NULL);
 
-  /* initialize the inverse temperatures */
-  pgd->rgInvTemperatures[0] = 0.4;                         /* hot */
+    /* allocate working arrays */
+    if (!(pgd->rgdlnPi = InitdVector (NTEMP)) ||
+        !(pgd->rglTemp = InitlVector (NTEMP)))
+      ReportError (NULL, RE_OUTOFMEM | RE_FATAL, "SetInvTemperatures", NULL);
 
-  for (i = 1; i < NTEMP - 1; i++)
-    pgd->rgInvTemperatures[i] = pow(0.8, (NTEMP - 1 - i)); /* medium... */
+    /* initialize the inverse temperatures */
+    pgd->rgInvTemperatures[0] = 0.4;                         /* hot */
 
-  pgd->rgInvTemperatures[NTEMP - 1] = 1;                   /* cold */
+    for (i = 1; i < NTEMP - 1; i++)
+      pgd->rgInvTemperatures[i] = pow(0.8, (NTEMP - 1 - i)); /* medium... */
+
+    pgd->rgInvTemperatures[NTEMP - 1] = 1;                   /* cold */
+
+  } /* end if */
 
 } /* SetInvTemperatures */
 
@@ -411,75 +471,23 @@ void DoMarkov (PANALYSIS panal)
 {
   PGIBBSDATA pgd = &panal->gd;
   PLEVEL     pLevel0 = panal->pLevels[0];
-  char       c;
   long       nThetas, nUpdateAt, nTotal;
   long       i, iter = 0;
   long       nIter = pgd->nMaxIter;  /* scheduled iterations of the sampler */
   double     *pdMCVarVals  = NULL;   /* read in values of thetas */
   double     *pdSum        = NULL;   /* column sums of read thetas */
   double     **prgdSumProd = NULL;   /* column sums of thetas cross-products */
-  double     dLnPrior = 0, dLnData = 0;
+  double     dTmp, dLnPrior = 0, dLnData = 0;
 
   InitRandom (panal->dSeed, TRUE);
 
-  switch (pgd->nSimTypeFlag) {
+  AnnounceMarkov (pgd->nSimTypeFlag, nIter);
 
-  case 0:
-    printf ("\nDoing %ld Metropolis within Gibbs simulation",
-            nIter);
-    printf ((nIter != 1 ? "s\n" : "\n"));
-    break;
-
-  case 1:
-    printf ("\nPrinting data and predictions for the last line of the "
-            "restart file\n");
-    break;
-
-  case 2:
-    printf ("\nDoing %ld Metropolis simulation",
-            nIter);
-    printf ((nIter != 1 ? "s\n" : "\n"));
-    break;
-
-  case 3:
-    printf ("\nDoing %ld Metropolis within Gibbs tempered simulation",
-            nIter);
-    printf ((nIter != 1 ? "s\n" : "\n"));
-
-    /* inverse temperatures not set by the user: default */
-    if (pgd->nInvTemperatures == 0)
-      SetInvTemperatures (pgd);
-
-    break;
-
-  case 4:
-    printf("\nDoing Stochastic optimization...\n");
-    break;
-  }
+  SetInvTemperatures (pgd); // (if needed)
 
   OpenMarkovFiles (panal);
 
-  if (panal->gd.szGdata) { /* read the data from a file */
-    FILE *pfile = fopen (panal->gd.szGdata, "r");
-    if (!pfile) {
-      printf ("Cannot open data file '%s'\n", panal->gd.szGdata);
-      exit (0);
-    } /* if */
-
-    /* skip the first line */
-    do { c = getc(pfile); } while (c != '\n');
-
-    TraverseLevels (pLevel0, ReadData, pfile, NULL);
-
-    fclose (pfile);
-  }
-
-  /* we do not want more than 1 top level; Get out;
-     This could be dealth with earlier, though */
-  if (panal->iInstances > 1) {
-    printf("Error: only one top level is allowed\nExiting\n");
-    exit(0);
-  }
+  ReadDataFile (panal);     // (if needed)
 
   /* MC variables must be placed in arrays at the next lower level */
   TraverseLevels (pLevel0, CloneMCVars, NULL);
@@ -488,7 +496,7 @@ void DoMarkov (PANALYSIS panal)
   TraverseLevels (pLevel0, CloneLikes, NULL);
 
   /* Find the parents and dependents of the MC vars */
-  TraverseLevels (pLevel0, FindMCParents, panal, NULL);
+  TraverseLevels (pLevel0, FindMCParents,    panal, NULL);
   TraverseLevels (pLevel0, FindMCDependents, panal, NULL);
 
   /* Find the parents of the parameters in likelihood statements */
@@ -499,9 +507,9 @@ void DoMarkov (PANALYSIS panal)
   if (pgd->nSimTypeFlag != 1) {
     fprintf (pgd->pfileOut, "iter\t");
     TraverseLevels (pLevel0, WriteHeader, panal, pgd->pfileOut, NULL);
-    if (pgd->nSimTypeFlag == 3) {   /* tempered */
+    if ((pgd->nSimTypeFlag == 3) || (pgd->nSimTypeFlag == 4)) { /* tempered */
       fprintf (pgd->pfileOut, "IndexT\t");
-      for (i = 0; i < pgd->nInvTemperatures; i++) 
+      for (i = 0; i < pgd->nInvTemperatures; i++)
         fprintf (pgd->pfileOut, "LnPseudoPrior(%ld)\t",i);
       fprintf (pgd->pfileOut, "LnPrior\tLnData\tLnPosterior\n");
     }
@@ -522,7 +530,7 @@ void DoMarkov (PANALYSIS panal)
      for identical 'Print' statements */
   TraverseLevels (pLevel0, CheckPrintStatements, panal, NULL);
 
-  /* Print out the structure for checking */
+  /* Print out the structure for debugging */
   if (panal->bDependents) {
     TraverseLevels (pLevel0, PrintDeps, NULL);
     CloseMarkovFiles (panal);
@@ -554,7 +562,7 @@ void DoMarkov (PANALYSIS panal)
     /* Read the starting values in the order they are printed and
        close the file when finished */
 
-    if (pgd->nSimTypeFlag == 3)
+    if ((pgd->nSimTypeFlag == 3) || (pgd->nSimTypeFlag == 4))
       ReadRestartTemper (pgd->pfileRestart, nThetas, pgd->nInvTemperatures,
                          pdMCVarVals, pdSum, prgdSumProd, &iter,
                          &pgd->indexT, pgd->rgdlnPi);
@@ -586,7 +594,7 @@ void DoMarkov (PANALYSIS panal)
       FILE *pfile = fopen (szKernelFile, "r");
 
       if (pfile) {
-        printf("Reading kernel file %s\n", szKernelFile); 
+        printf("Reading kernel file %s\n", szKernelFile);
         TraverseLevels (pLevel0, ReadKernel, pfile, NULL);
       }
       else {
@@ -622,21 +630,22 @@ void DoMarkov (PANALYSIS panal)
     /* Set the jumping kernel's SD */
     TraverseLevels (pLevel0, SetKernel, 2, pdMCVarVals, NULL);
 
-    /* Initialize the thetas by sampling, write them out at the same time */
+    /* Initialize the thetas by sampling from the prior,
+       and write them out at the same time */
     fprintf (pgd->pfileOut, "0\t");
     TraverseLevels (pLevel0, InitMCVars, pgd->pfileOut, NULL);
 
     /* Calculate the total prior */
     TraverseLevels (pLevel0, CalculateTotals, panal, &dLnPrior, NULL);
-    /* Initilalize the predictions arrays by running all experiments and
-       save the data likelihood output */
+    /* Initialize the predictions arrays by running all experiments */
     if (!RunAllExpts (panal, &dLnData)) {
-      /* error, cannot compute */
+      /* Error, cannot compute */
       printf ("\nError: cannot compute at the starting point - Exiting\n\n");
       exit(0);
     }
 
-    if (pgd->nSimTypeFlag == 3) { /*output indexT and pseudo-priors */
+    if ((pgd->nSimTypeFlag == 3) || (pgd->nSimTypeFlag == 4)) {
+      /* Output indexT and pseudo-priors */
       fprintf (pgd->pfileOut, "0\t");
       for (i = 0; i < pgd->nInvTemperatures; i++)
         fprintf (pgd->pfileOut, "%e\t", pgd->rgdlnPi[i]);
@@ -660,15 +669,14 @@ void DoMarkov (PANALYSIS panal)
 
     /* Output to screen, eventually */
     if (panal->bPrintIter && ((iter+1) % 100 == 0))
-      fprintf(stderr, "Iteration %ld\n", iter + 1);
+      printf("Iteration %ld\n", iter + 1);
 
     /* Start output to file, eventually */
-
     if (((iter + 1) % pgd->nPrintFreq == 0) &&
         (iter >= pgd->nMaxIter - pgd->nPrintIter))
       fprintf (pgd->pfileOut, "%ld\t", iter + 1);
 
-    if ((pgd->nSimTypeFlag == 0) || (pgd->nSimTypeFlag == 4)) { 
+    if ((pgd->nSimTypeFlag == 0) || (pgd->nSimTypeFlag == 5)) {
       /* component by component jumps or stochastic optimization */
 
       TraverseLevels (pLevel0, SampleThetas, panal, pgd, &iter, &nUpdateAt,
@@ -688,9 +696,9 @@ void DoMarkov (PANALYSIS panal)
     }
     else {
 
-      if (pgd->nSimTypeFlag == 3) { /* tempered */
-
-        TraverseLevels (pLevel0, SampleThetasTempered, panal, pgd, &iter, 
+      if ((pgd->nSimTypeFlag == 3) || (pgd->nSimTypeFlag == 4)) {
+        /* tempered */
+        TraverseLevels (pLevel0, SampleThetasTempered, panal, pgd, &iter,
                         &nUpdateAt,  &nTotal, &pgd->indexT, NULL);
 
         dLnPrior = 0.0;
@@ -698,26 +706,20 @@ void DoMarkov (PANALYSIS panal)
         dLnData = 0.0;
         TraverseLevels1 (pLevel0, SumAllExpts, &dLnData, NULL);
 
-        /* update temperature jump counter */
-        pgd->attemptedTempe++; 
-
         /* Robbins-Monro updating of the pseudo prior */
-        for (i = 0; i <pgd->nInvTemperatures; i++)
+        for (i = 0; i < pgd->nInvTemperatures; i++) {
+          dTmp = pgd->dCZero / (double) (iter + pgd->dNZero);
           if (i == pgd->indexT)
-            pgd->rgdlnPi[i] = pgd->rgdlnPi[i] - 
-                              (pgd->dCZero / 
-                               (pgd->attemptedTempe + pgd->dNZero));
+            pgd->rgdlnPi[i] -= dTmp;
           else
-            pgd->rgdlnPi[i] = pgd->rgdlnPi[i] + 
-                              (pgd->dCZero / 
-                               (pgd->nInvTemperatures * 
-                                (pgd->attemptedTempe + pgd->dNZero)));
+            pgd->rgdlnPi[i] += dTmp / (double) pgd->nInvTemperatures;
+        }
 
         /* Output indexT and log-densities, eventually */
         if (((iter + 1) % pgd->nPrintFreq == 0) &&
             (iter >= pgd->nMaxIter - pgd->nPrintIter)) {
           fprintf (pgd->pfileOut, "%ld\t", pgd->indexT);
-          for (i = 0; i < pgd->nInvTemperatures; i++) 
+          for (i = 0; i < pgd->nInvTemperatures; i++)
             fprintf (pgd->pfileOut, "%e\t", pgd->rgdlnPi[i]);
           fprintf (pgd->pfileOut, "%e\t%e\t%e\n", dLnPrior, dLnData,
                    dLnPrior + dLnData);
@@ -749,7 +751,7 @@ void DoMarkov (PANALYSIS panal)
         }
       }
     }
-    /* Adjust the update time eventually */
+    /* Adjust the update time, eventually */
     if (iter == nUpdateAt) {
       nTotal = nTotal * 3 / 2;
       nUpdateAt = iter + nTotal;
@@ -782,7 +784,7 @@ void DoMarkov (PANALYSIS panal)
 
   CloseMarkovFiles (panal);
 
-  if (pgd->nSimTypeFlag == 3){
+  if ((pgd->nSimTypeFlag == 3) || (pgd->nSimTypeFlag == 4)) {
     printf ("\nTemperature count:\n");
     for (i = 0; i < pgd->nInvTemperatures - 1; i++)
       printf ("%ld\t", pgd->rglTemp[i]);
@@ -828,7 +830,7 @@ void FindLikeParents (PLEVEL plevel, char **args)
             bFound = TRUE;
           }
         } /* for m */
-        
+
         /* Now, all previous levels */
         if (!bFound) {
           for (n = plevel->iDepth - 1; n >= 0; n--) {
@@ -848,7 +850,7 @@ void FindLikeParents (PLEVEL plevel, char **args)
           printf ("\n"
                   "Error: parent in position %ld of %s must be\n"
                   "       declared before it when creating\n"
-                  "       sampling dependencies - Exiting.\n\n", 
+                  "       sampling dependencies - Exiting.\n\n",
                   l, pMCVar1->pszName);
           exit(0);
         }
@@ -919,7 +921,7 @@ void FindMCParents (PLEVEL plevel, char **args)
             bFound = TRUE;
           }
         } /* for m */
-        
+
         /* Now, all previous levels */
         if (!bFound) {
           for (n = plevel->iDepth - 1; n >= 0; n--) {
@@ -939,7 +941,7 @@ void FindMCParents (PLEVEL plevel, char **args)
           printf ("\n"
                   "Error: parent in position %ld of %s must be\n"
                   "       declared before it when creating\n"
-                  "       sampling dependencies - Exiting.\n\n", 
+                  "       sampling dependencies - Exiting.\n\n",
                   l, pMCVar1->pszName);
           exit(0);
         }
@@ -1061,7 +1063,9 @@ void ListToPVArrayL (PVOID pData, PVOID pUser1, PVOID pUser2, PVOID pUser3)
 
    Returns the log of the (exact) density of variate under its distribution.
 */
-#define LNSQRT2PI 9.189385332046E-01
+#define LNSQRT2PI  0.918938533204672669541
+#define LNINVERPI -1.144729885849400163877
+#define LN2OVERPI -0.4515827052894548221396
 
 double LnDensity (PMCVAR pMCVar, PANALYSIS panal)
 {
@@ -1081,7 +1085,7 @@ double LnDensity (PMCVAR pMCVar, PANALYSIS panal)
       exit (0);
     }
   }
-  else if (pMCVar->iType == MCV_GENLOGNORMAL || 
+  else if (pMCVar->iType == MCV_GENLOGNORMAL ||
            pMCVar->iType == MCV_STUDENTT) {
     if (dParm1 < 0) {
       printf ("Error: parameter %g out of bounds in LnDensity\n",dParm1);
@@ -1218,9 +1222,9 @@ double LnDensity (PMCVAR pMCVar, PANALYSIS panal)
       return (dParm1 - 1) * log(dTheta) - dParm2 * dTheta +
              dParm1 * log(dParm2) - lnGamma (dParm1);
 
-    case MCV_TRUNCINVGGAMMA: 
+    case MCV_TRUNCINVGGAMMA:
 #ifdef HAVE_LIBGSL
-      dTmp = gsl_cdf_gamma_P (1/dMax, dParm1, dParm2) - 
+      dTmp = gsl_cdf_gamma_P (1/dMax, dParm1, dParm2) -
              gsl_cdf_gamma_P (1/dMin, dParm1, dParm2); /* fall thru */
 #else
       printf ("Error: Truncated inverse gamma density cannot be evaluated\n");
@@ -1282,12 +1286,12 @@ double LnDensity (PMCVAR pMCVar, PANALYSIS panal)
       }
       /* This is relative stdev of lognormal part -- dMin is sigma for the
          lognormal part */
-      dTmp = sqrt(exp(pow(dMin,2)) * (exp(pow(dMin,2)) - 1)); 
+      dTmp = sqrt(exp(pow(dMin,2)) * (exp(pow(dMin,2)) - 1));
       /* This is the transformation parameter lambda */
       dTmp2 = pow(dParm2/dTmp,2);
       /* This is the transformed mean */
       dTmp3 = log(dParm1 + sqrt(pow(dParm1,2) + dTmp2));
-      /* This is the transformed theta -- use Taylor series for 
+      /* This is the transformed theta -- use Taylor series for
          negative dTheta when lambda/dTheta^2 < 0.01 */
       if ((dTheta < 0) && (dTmp2 < (0.01*dTheta*dTheta)))
         dTmp4 = log(dTmp2/(-2*dTheta)*(1+0.25*dTmp2/pow(dTheta,2)));
@@ -1308,8 +1312,10 @@ double LnDensity (PMCVAR pMCVar, PANALYSIS panal)
               -dTmp * log(1 + pow((dTheta - dParm2) / dMin, 2) / dParm1));
 
     case MCV_CAUCHY:
+      return (LNINVERPI - log(dParm1 + dTheta * dTheta / dParm1));
+
     case MCV_HALFCAUCHY: /* dParm1 is scale */
-      return (- log(PI * (dParm1 + dTheta * dTheta / dParm1)));
+      return (LN2OVERPI - log(dParm1 + dTheta * dTheta / dParm1));
 
     default:
       ReportRunTimeError(panal, RE_UNKNOWNDIST | RE_FATAL, "LnDensity");
@@ -1458,7 +1464,7 @@ double MinMCVar (PMCVAR pMCVar) {
 
   /* if the parameter has a discrete distribution, round it - FB 12/06/97 */
   if (pMCVar->iType == MCV_BINOMIAL || pMCVar->iType == MCV_POISSON) {
-   return( *(pMCVar->pdParm[2]));                                 
+   return( *(pMCVar->pdParm[2]));
   }
   else { /* FB fixed the uniform case - FB 30/06/97 */
     if (pMCVar->iType == MCV_UNIFORM || pMCVar->iType == MCV_LOGUNIFORM) {
@@ -1605,8 +1611,8 @@ int PrintExpt (PLEVEL plevel, char **args)
 /* Function -------------------------------------------------------------------
    ReadData
 
-   Reads in data from a specified data file. There need to be one data item
-   per output specified, and in the order given by the Print or PrintStep 
+   Reads data for an experiment. There need to be one data item
+   per output specified, and in the order given by the Print or PrintStep
    statements.
    Upgraded by FB 27/03/99 to deal with improved data handling.
    Called from TraverseLevels.
@@ -1659,6 +1665,36 @@ void ReadData (PLEVEL plevel, char **args)
 } /* ReadData */
 
 
+/* Function -------------------------------------------------------------------
+   ReadDataFile
+
+   Reads in data from a specified data file, if specified.
+*/
+
+void ReadDataFile (PANALYSIS panal)
+{
+  if (panal->gd.szGdata) {
+
+    char c;
+
+    FILE *pfile = fopen (panal->gd.szGdata, "r");
+
+    if (!pfile) {
+      printf ("Cannot open data file '%s'\n", panal->gd.szGdata);
+      exit (0);
+    }
+
+    /* skip the first line */
+    do { c = getc(pfile); } while (c != '\n');
+
+    TraverseLevels (panal->pLevels[0], ReadData, pfile, NULL);
+
+    fclose (pfile);
+  }
+
+} /* ReadDataFile */
+
+
 /* Function -------------------------------------------------------------------
    PrintDeps
 
@@ -1672,30 +1708,28 @@ void PrintDeps (PLEVEL plevel, char **args)
   long n, m;
   PMCVAR pMCVar;
 
-  fprintf (stderr, "Depth %d; Instance %d\n",
-           plevel->iDepth, plevel->iSequence);
+  printf ("Depth %d; Instance %d\n", plevel->iDepth, plevel->iSequence);
 
   for (n = 0; n < plevel->nMCVars; n++) {
     pMCVar = plevel->rgpMCVars[n];
 
-    fprintf(stderr, "Variable %s (%d) [%lx]\n",
-            pMCVar->pszName, pMCVar->iDepth, (unsigned long) pMCVar);
+    printf ("Variable %s (%d) [%" PRIxPTR "]\n",
+            pMCVar->pszName, pMCVar->iDepth, (intptr_t) pMCVar);
 
     for (m = 0; m < 4; m++)
       if (pMCVar->pMCVParent[m] != NULL)
-        fprintf (stderr, "  Parent %ld: %s (%d) [%lx]\n", m,
-                 pMCVar->pMCVParent[m]->pszName,
-                 pMCVar->pMCVParent[m]->iDepth,
-                 (unsigned long) pMCVar->pMCVParent[m]);
+        printf ("  Parent %ld: %s (%d) [%" PRIxPTR "]\n", m,
+                pMCVar->pMCVParent[m]->pszName, pMCVar->pMCVParent[m]->iDepth,
+                (intptr_t) pMCVar->pMCVParent[m]);
 
     for (m = 0; m < pMCVar->nDependents; m++)
-      fprintf (stderr, "  Dependent: %s (%d) [%lx]\n",
-               pMCVar->rgpDependents[m]->pszName,
-               pMCVar->rgpDependents[m]->iDepth,
-               (unsigned long) pMCVar->rgpDependents[m]);
+      printf ("  Dependent: %s (%d) [%" PRIxPTR "]\n",
+              pMCVar->rgpDependents[m]->pszName,
+              pMCVar->rgpDependents[m]->iDepth,
+              (intptr_t) pMCVar->rgpDependents[m]);
 
     if (pMCVar->bExptIsDep)
-      fprintf(stderr, "  This variable influences experiments directly\n");
+      printf("  This variable influences experiments directly\n");
   }
 
 } /* PrintDeps */
@@ -1779,11 +1813,11 @@ void ReadRestartTemper (FILE *pfileRestart, long nThetas, int nInvTemperatures,
 {
   register char c;
   register long i, j;
-  double aux[nInvTemperatures];
+  double *pdAux;
   long iT;
   *pIter = -1;
 
-  for (i = 0; i < nInvTemperatures; i++) aux[i] = 0.0;
+  pdAux = InitdVector (nThetas);
 
   for (i = 0; i < nThetas; i++) {
     pdSum[i] = 0.0;
@@ -1818,7 +1852,7 @@ void ReadRestartTemper (FILE *pfileRestart, long nThetas, int nInvTemperatures,
     }
 
     for (i = 0; i < nInvTemperatures; i++) {
-      if (fscanf(pfileRestart,"%lg", &(aux[i])) == EOF) {
+      if (fscanf(pfileRestart,"%lg", &(pdAux[i])) == EOF) {
         printf ("Error: incorrect length for restart file - Exiting\n");
         exit(0);
       }
@@ -1829,8 +1863,8 @@ void ReadRestartTemper (FILE *pfileRestart, long nThetas, int nInvTemperatures,
     do { c = getc(pfileRestart); } while (c != '\n');
 
     *pindexT = iT;
-    for (i = 0; i < nInvTemperatures; i++) 
-      pdlnPi[i] = aux[i];
+    for (i = 0; i < nInvTemperatures; i++)
+      pdlnPi[i] = pdAux[i];
 
     /* update prgdSumProd */
     for (i = 0; i < nThetas; i++)
@@ -1845,6 +1879,7 @@ void ReadRestartTemper (FILE *pfileRestart, long nThetas, int nInvTemperatures,
   /* note that the theta returned is the last parameter set read */
 
   fclose (pfileRestart);
+  free (pdAux);
 
 } /* ReadRestartTemper */
 
@@ -1871,7 +1906,7 @@ int RestoreLikelihoods (PLEVEL plevel, char **args)
 /* Function -------------------------------------------------------------------
    RunAllExpts
 
-   Run all experiments (i.e. experiments at all levels below 
+   Run all experiments (i.e. experiments at all levels below
    panal->pLevels[0]).
 */
 
@@ -1960,13 +1995,12 @@ long SampleTemperature (PGIBBSDATA pgd, double dLnPrior, double dLnData)
   }
 
   /* Test the temperature */
-  if (TestTemper (pgd, indexT, indexT_new, dLnPrior, dLnData, 
-                  pgd->attemptedTempe, pgd->rgdlnPi[indexT], 
-                  pgd->rgdlnPi[indexT_new])) {   
+  if (TestTemper (pgd, indexT, indexT_new, dLnPrior, dLnData,
+                  pgd->rgdlnPi[indexT], pgd->rgdlnPi[indexT_new])) {
     /* jump */
     return (indexT_new);
   }
-  else 
+  else
     return (indexT);
 
 }  /* SampleTemperature */
@@ -2121,19 +2155,19 @@ void SampleThetas (PLEVEL plevel, char **args)
 
     } // end of kernel stuff
 
-    // Compute density under jump kernel
-    dLnKern  = log(CDFNormal ((MaxMCVar(pMCVar) - pMCVar->dVal) / 
+    // compute the integral of the jump kernel
+    dLnKern  = log(CDFNormal ((MaxMCVar(pMCVar) - pMCVar->dVal) /
                               pMCVar->dKernelSD) -
-                   CDFNormal ((MinMCVar(pMCVar) - pMCVar->dVal) / 
+                   CDFNormal ((MinMCVar(pMCVar) - pMCVar->dVal) /
                               pMCVar->dKernelSD));
 
     /* sample a new value */
     pMCVar->dVal = SampleTheta (pMCVar);
 
-    // update density under jump kernel
-    dLnKernNew  =  log(CDFNormal ((MaxMCVar(pMCVar) - pMCVar->dVal) / 
+    // update the integral of the jump kernel
+    dLnKernNew  =  log(CDFNormal ((MaxMCVar(pMCVar) - pMCVar->dVal) /
                                   pMCVar->dKernelSD) -
-                       CDFNormal ((MinMCVar(pMCVar) - pMCVar->dVal) / 
+                       CDFNormal ((MinMCVar(pMCVar) - pMCVar->dVal) /
                                   pMCVar->dKernelSD));
 
     // update prior and likelihood
@@ -2157,7 +2191,7 @@ void SampleThetas (PLEVEL plevel, char **args)
 
     /* Test the results and act accordingly */
     if (!TestImpRatio (pgd, pMCVar->bExptIsDep, dLnKern, dLnKernNew,
-                       dLnPrior, dLnPriorNew, 
+                       dLnPrior, dLnPriorNew,
                        dLnLike, dLnLikeNew, dLnData, dLnDataNew)) {
       pMCVar->dVal = dTheta;
 
@@ -2171,7 +2205,7 @@ void SampleThetas (PLEVEL plevel, char **args)
         TraverseLevels1 (plevel, SaveLikelihoods, NULL);
     }
 
-WriteIt: /* Write the MC var value to output file */
+    WriteIt: /* Write the MC var value to output file */
 
     if (((*pnIter+1) % pgd->nPrintFreq == 0) &&
         (*pnIter >= pgd->nMaxIter - pgd->nPrintIter)) {
@@ -2301,7 +2335,7 @@ void SampleThetasTempered (PLEVEL plevel, char **args)
     /* sample a new value */
 
     if (pgd->rgInvTemperatures[*pindexT] > 0) { // usual case
-    
+
       /* first scale temporarily the kernelSD according to the temperature */
 
       /* save the current kernelSD */
@@ -2309,9 +2343,9 @@ void SampleThetasTempered (PLEVEL plevel, char **args)
 
       /* update the kernelSD according to the temperature, the formula
          comes from that of the variance of the powered normal */
-      pMCVar->dKernelSD = pow(2 * PI, 
+      pMCVar->dKernelSD = pow(2 * PI,
                               0.5 * (1 - pgd->rgInvTemperatures[*pindexT])) *
-                          pow(pMCVar->dKernelSD, 
+                          pow(pMCVar->dKernelSD,
                               3 - pgd->rgInvTemperatures[*pindexT]) /
                           pow(pgd->rgInvTemperatures[*pindexT], 1.5);
 
@@ -2319,30 +2353,35 @@ void SampleThetasTempered (PLEVEL plevel, char **args)
       if (pMCVar->dKernelSD > pMCVar->dMaxKernelSD)
         pMCVar->dKernelSD = pMCVar->dMaxKernelSD;
 
-      // Compute density under jump kernel
-      dLnKern  = log(CDFNormal ((MaxMCVar(pMCVar) - pMCVar->dVal) / 
+      // compute the integral of the jump kernel
+      dLnKern  = log(CDFNormal ((MaxMCVar(pMCVar) - pMCVar->dVal) /
                                 pMCVar->dKernelSD) -
-                     CDFNormal ((MinMCVar(pMCVar) - pMCVar->dVal) / 
+                     CDFNormal ((MinMCVar(pMCVar) - pMCVar->dVal) /
                                 pMCVar->dKernelSD));
 
       /* sample a new value */
       pMCVar->dVal = SampleTheta (pMCVar);
 
-      // update density under jump kernel
-      dLnKernNew  =  log(CDFNormal ((MaxMCVar(pMCVar) - pMCVar->dVal) / 
+      // update the integral of the jump kernel
+      dLnKernNew  =  log(CDFNormal ((MaxMCVar(pMCVar) - pMCVar->dVal) /
                                     pMCVar->dKernelSD) -
-                         CDFNormal ((MinMCVar(pMCVar) - pMCVar->dVal) / 
+                         CDFNormal ((MinMCVar(pMCVar) - pMCVar->dVal) /
                                     pMCVar->dKernelSD));
-    } 
-    else { // inverse temperature is zero, sample uniformly
+    }
+    else { // inverse temperature is zero, sample uniformly or from the prior
       dLnKern = dLnKernNew  = 1;
-      // sample a new value uniformly on its range
-      pMCVar->dVal = SampleThetaUnif (pMCVar);
+      if (pgd->nSimTypeFlag == 3) { // the posterior is tempered
+        // sample a new value uniformly in its range
+        pMCVar->dVal = SampleThetaUnif (pMCVar);
+      }
+      else { // the likelihood is tempered, sample from the prior
+        CalculateOneMCParm (pMCVar);
+      }
     }
 
     /* recompute prior and likelihood */
     dLnPriorNew = LnDensity (pMCVar, panal);
-    dLnLikeNew = LnLike (pMCVar, panal);
+    dLnLikeNew  = LnLike (pMCVar, panal);
 
     dLnDataNew = 0.0;
 
@@ -2361,7 +2400,7 @@ void SampleThetasTempered (PLEVEL plevel, char **args)
     /* Test the results and act accordingly */
     if (!TestImpRatioTemper (pgd, pMCVar->bExptIsDep, dLnKern, dLnKernNew,
                              dLnPrior, dLnPriorNew,
-                             dLnLike, dLnLikeNew, dLnData, dLnDataNew, 
+                             dLnLike, dLnLikeNew, dLnData, dLnDataNew,
                              *pindexT)) {
       pMCVar->dVal = dTheta;
        if(pMCVar->bExptIsDep)
@@ -2490,11 +2529,11 @@ void SampleThetaVector (PLEVEL pLevel, PANALYSIS panal, long nThetas,
 
   /* Set the dVals of the variables to the values sampled and check that
      we are within bounds */
-  nThetas = 0;
-  bInBounds = TraverseLevels1 (pLevel, SetMCVars, pdTheta, &nThetas, NULL);
+  long iTmp = 0; // dummy variable to avoid resetting nThetas FB 16/07/2016
+  bInBounds = TraverseLevels1 (pLevel, SetMCVars, pdTheta, &iTmp, NULL);
 
   if (!bInBounds) { // reject
-    for (i = 0; i < nThetas; i++) 
+    for (i = 0; i < nThetas; i++)
       pdTheta[i] = pdTheta_old[i]; // restore
     goto DontJump;
   }
@@ -2527,7 +2566,7 @@ void SampleThetaVector (PLEVEL pLevel, PANALYSIS panal, long nThetas,
     }
   }
 
- DontJump:
+  DontJump:
 
   /* update arrays */
   for (i = 0; i < nThetas; i++) {
@@ -2589,7 +2628,7 @@ void SetFixedVars (PLEVEL plevel)
 
 void SetKernel (PLEVEL plevel, char **args)
 {
-  long useMCVarVals = (long) args[0]; /* 1 if dVal is to be restored, else 2 */
+  intptr_t useMCVarVals = (intptr_t) args[0]; /* 1 to restore dVal, else 2 */
   double *pdMCVarVals = (double *) args[1];
   double dMin, dMax, dTmp;
   long   n, m;
@@ -2599,16 +2638,18 @@ void SetKernel (PLEVEL plevel, char **args)
   /* set the jumping kernel's SD: sample 4 variates and take the range */
   for (n = 0; n < plevel->nMCVars; n++) {
 
-    if ( !(plevel->rgpMCVars[n]->bIsFixed)) {
+    if (!(plevel->rgpMCVars[n]->bIsFixed)) {
 
       pMCVar = plevel->rgpMCVars[n];
       CalculateOneMCParm (pMCVar);
 
       /* set the maximum kernel SD value */
       if (pMCVar->iType == MCV_UNIFORM || pMCVar->iType == MCV_LOGUNIFORM )
-        pMCVar->dMaxKernelSD = *(pMCVar->pdParm[1]) / 6.0 - *(pMCVar->pdParm[0]) / 6.0;
+        pMCVar->dMaxKernelSD = (*(pMCVar->pdParm[1]) - *(pMCVar->pdParm[0])) /
+                               6.0;
       else
-        pMCVar->dMaxKernelSD = *(pMCVar->pdParm[3]) / 6.0 - *(pMCVar->pdParm[2]) / 6.0;
+        pMCVar->dMaxKernelSD = (*(pMCVar->pdParm[3]) - *(pMCVar->pdParm[2])) /
+                               6.0;
 
       /* sample 4 variates */
       dMin = dMax = pMCVar->dVal;
@@ -2626,8 +2667,8 @@ void SetKernel (PLEVEL plevel, char **args)
       else
         pMCVar->dKernelSD = dMax - dMin;
 
-      /* take care of the case in which the range is zero (can happens for discrete 
-         variables) */
+      /* take care of the case in which the range is zero
+         (can happens for discrete variables) */
       if (pMCVar->dKernelSD == 0) {
         pMCVar->dKernelSD = pMCVar->dMaxKernelSD;
       }
@@ -2887,7 +2928,7 @@ int SumAllExpts (PLEVEL plevel, char **args)
    Test prior, likelihood against random number between 0 and 1
 */
 
-BOOL TestImpRatio (PGIBBSDATA pgd,  BOOL bExptIsDep, 
+BOOL TestImpRatio (PGIBBSDATA pgd,  BOOL bExptIsDep,
                    double dLnKern,  double dLnKernNew,
                    double dLnPrior, double dLnPriorNew,
                    double dLnLike,  double dLnLikeNew,
@@ -2901,7 +2942,7 @@ BOOL TestImpRatio (PGIBBSDATA pgd,  BOOL bExptIsDep,
       dLnDataNew  == NULL_SUPPORT)
     return FALSE;
 
-  dPjump = dLnPriorNew - dLnPrior + dLnLikeNew - dLnLike + 
+  dPjump = dLnPriorNew - dLnPrior + dLnLikeNew - dLnLike +
            dLnKern - dLnKernNew;
 
   if (bExptIsDep)
@@ -2910,10 +2951,10 @@ BOOL TestImpRatio (PGIBBSDATA pgd,  BOOL bExptIsDep,
   if (pgd->nSimTypeFlag == 0)
     return ((BOOL) (log(Randoms()) <= dPjump));
   else {
-    if (pgd->nSimTypeFlag == 4)
+    if (pgd->nSimTypeFlag == 5)
       return ((BOOL) (0 <= dPjump));
     else {
-      printf ("Error: SimTypeFlag should be 0 or 4 in TestImpRatio "
+      printf ("Error: simTypeFlag should be 0 or 5 in TestImpRatio "
               "- Exiting.\n\n");
       exit (0);
     }
@@ -2925,7 +2966,7 @@ BOOL TestImpRatio (PGIBBSDATA pgd,  BOOL bExptIsDep,
 /* Function -------------------------------------------------------------------
    TestImpRatioTemper
 
-   Test prior, likelihood against a random number between 0 and 1 according to 
+   Test prior, likelihood against a random number between 0 and 1 according to
    the temperature
 */
 
@@ -2942,12 +2983,19 @@ BOOL TestImpRatioTemper (PGIBBSDATA pgd,  BOOL bExptIsDep,
       dLnDataNew  == NULL_SUPPORT)
     return FALSE;
 
-  dPjump = pgd->rgInvTemperatures[indexT] * 
-           (dLnPriorNew - dLnPrior + dLnLikeNew - dLnLike + 
-            dLnKern - dLnKernNew);
+  if (pgd->nSimTypeFlag == 3) { // posterior is tempered
+    dPjump = pgd->rgInvTemperatures[indexT] *
+             (dLnPriorNew - dLnPrior + dLnLikeNew - dLnLike +
+              dLnKern - dLnKernNew);
+  }
+  else { // only the likelihood is tempered
+    dPjump = dLnPriorNew - dLnPrior +
+             pgd->rgInvTemperatures[indexT] *
+             (dLnLikeNew - dLnLike + dLnKern - dLnKernNew);
+  }
 
   if (bExptIsDep)
-    dPjump += pgd->rgInvTemperatures[indexT] *(dLnDataNew - dLnData);
+    dPjump += pgd->rgInvTemperatures[indexT] * (dLnDataNew - dLnData);
 
   return ((BOOL) (log(Randoms()) <= dPjump));
 
@@ -2960,9 +3008,8 @@ BOOL TestImpRatioTemper (PGIBBSDATA pgd,  BOOL bExptIsDep,
    Test temperature against a random number between 0 and 1
 */
 
-BOOL TestTemper (PGIBBSDATA pgd, long indexT, long indexT_new, double dLnPrior, 
-                 double dLnData, long attemptedTempe,
-                 double pseudo, double pseudonew)
+BOOL TestTemper (PGIBBSDATA pgd, long indexT, long indexT_new, double dLnPrior,
+                 double dLnData, double pseudo, double pseudonew)
 {
   double dPjump = 0;
   #define MINUSLN2  -0.6931471805599452862268
@@ -2970,13 +3017,24 @@ BOOL TestTemper (PGIBBSDATA pgd, long indexT, long indexT_new, double dLnPrior,
   if (dLnPrior + dLnData == NULL_SUPPORT)
     return FALSE;
 
-  dPjump = (pgd->rgInvTemperatures[indexT_new] -
-            pgd->rgInvTemperatures[indexT]) * (dLnPrior + dLnData) +
-           pseudonew - pseudo +
-           ((indexT_new == 0) || (indexT_new == pgd->nInvTemperatures - 1) ? 
-            0 : MINUSLN2) -
-           ((indexT     == 0) || (indexT     == pgd->nInvTemperatures - 1) ? 
-            0 : MINUSLN2);
+  if (pgd->nSimTypeFlag == 3) { /* the posterior is tempered */
+    dPjump = (pgd->rgInvTemperatures[indexT_new] -
+              pgd->rgInvTemperatures[indexT]) * (dLnPrior + dLnData) +
+             pseudonew - pseudo +
+             ((indexT_new == 0) || (indexT_new == pgd->nInvTemperatures - 1) ?
+              0 : MINUSLN2) -
+             ((indexT     == 0) || (indexT     == pgd->nInvTemperatures - 1) ?
+              0 : MINUSLN2);
+  }
+  else { /* only the likelihood is tempered */
+    dPjump = (pgd->rgInvTemperatures[indexT_new] -
+              pgd->rgInvTemperatures[indexT]) * dLnData +
+             pseudonew - pseudo +
+             ((indexT_new == 0) || (indexT_new == pgd->nInvTemperatures - 1) ?
+              0 : MINUSLN2) -
+             ((indexT     == 0) || (indexT     == pgd->nInvTemperatures - 1) ?
+              0 : MINUSLN2);
+  }
 
   return ((BOOL) (log(Randoms()) <= dPjump));
 
