@@ -78,7 +78,6 @@ void   CloneMCVars (PLEVEL plevel, char **args);
 void   CloneMCVarsL (PVOID pData, PVOID pUser1, PVOID pUser2, PVOID pUser3);
 void   CloseMarkovFiles (PANALYSIS panal);
 void   ConvertLists (PLEVEL plevel, char **args);
-double dInvTemper( long indexT);
 void   DoMarkov (PANALYSIS panal);
 void   FindLikeParents (PLEVEL plevel, char **args);
 void   FindMCDependents(PLEVEL plevel, char **args);
@@ -103,6 +102,8 @@ void   PrintAllExpts (PLEVEL plevel, PANALYSIS panal, PFILE pOutFile);
 void   PrintDeps (PLEVEL plevel, char **args);
 int    PrintExpt (PLEVEL plevel, char **args);
 void   ReadData (PLEVEL plevel, char **args);
+void   ReadDataFile (PANALYSIS panal);
+void   ReadKernel (PLEVEL plevel, char **args);
 void   ReadRestart (FILE *pfileRestart, long nThetas, double *pdTheta, 
                     double *pdSum, double **prgdSumProd, long *pIter);
 void   ReadRestartTemper (FILE *pfileRestart, long nThetas, 
@@ -122,13 +123,11 @@ void   SampleThetaVector (PLEVEL pLevel, PANALYSIS panal, long nThetas,
                           PDOUBLE pdLnPrior, PDOUBLE pdLnData);
 int    SaveLikelihoods (PLEVEL plevel, char **args);
 void   SetFixedVars (PLEVEL plevel);
-void   SetKernel (PLEVEL plevel, char **args);
-void   ReadKernel (PLEVEL plevel, char **args);
-void   WriteKernel (PLEVEL plevel, char **args);
-void   SetModelVars (PLEVEL plevel);
 int    SetMCVars (PLEVEL plevel, char **args);
-int    SumAllExpts (PLEVEL plevel, char **args);
+void   SetModelVars (PLEVEL plevel);
+void   SetKernel (PLEVEL plevel, char **args);
 void   SetPointers (PLEVEL plevel, char **args);
+int    SumAllExpts (PLEVEL plevel, char **args);
 double Temper_probabilities(int n, long i, long j);
 BOOL   TestImpRatio (PGIBBSDATA pgd,  BOOL bExptIsDep,
                      double dLnKern,  double dLnKernNew,
@@ -141,13 +140,14 @@ BOOL   TestImpRatioTemper (PGIBBSDATA pgd,  BOOL bExptIsDep,
                            double dLnLike,  double dLnLikeNew,
                            double dLnData,  double dLnDataNew, long indexT);
 BOOL   TestTemper (PGIBBSDATA pgd, long indexT, long indexT_new, 
-                   double dLnPrior, double dLnData, long attemptedTempe, 
-                   double pseudo, double pseudonew);
+                   double dLnPrior, double dLnData, double pseudo,
+                   double pseudonew);
 void   TraverseLevels (PLEVEL plevel,
                        void (*routinePtr)(PLEVEL plevel, char **args), ...);
 int    TraverseLevels1 (PLEVEL plevel,
                         int (*routinePtr)(PLEVEL plevel, char **args), ...);
 void   WriteHeader (PLEVEL plevel, char **args);
+void   WriteKernel (PLEVEL plevel, char **args);
 void   WriteMCVars (PLEVEL plevel, PFILE pOutFile);
 
 #define MH_H_DEFINED
