@@ -371,7 +371,7 @@ BOOL GetSimType (PINPUTBUF pibIn)
    temperature schedule.
    Return TRUE if the structure is defined, FALSE on error.  The
    command syntax includes the number of inverse temperatures then the list of
-   them (numbers in ]0,1] interval).
+   them (numbers in [0,inf] interval).
 */
 
 BOOL GetInvTemperature (PINPUTBUF pibIn, PSTR szLex, PGIBBSDATA pgd)
@@ -423,11 +423,12 @@ BOOL GetInvTemperature (PINPUTBUF pibIn, PSTR szLex, PGIBBSDATA pgd)
     /* initialize */
     pgd->rgdlnPi[i] = 0;
     pgd->rglTemp[i] = 0; /* temperature count*/
-    if ((pgd->rgInvTemperatures[i] < 0) || (pgd->rgInvTemperatures[i] > 1))
-      ReportError (pibIn, RE_LEXEXPECTED | RE_FATAL, "number in [0,1]", szLex);
-    if ((i > 0) && (pgd->rgInvTemperatures[i] <= pgd->rgInvTemperatures[i-1]))
+    if (pgd->rgInvTemperatures[i] < 0)
       ReportError (pibIn, RE_LEXEXPECTED | RE_FATAL, 
-                   "lower number", szLex);
+                   "positive inverse temperature", szLex);
+    if ((i > 0) && (pgd->rgInvTemperatures[i] <= pgd->rgInvTemperatures[i-1]))
+      ReportError (pibIn, RE_SPECERR | RE_FATAL, 
+                   "Inverse temperatures out of order", NULL);
   } /* for */
 
   bErr = EGetPunct (pibIn, szLex, CH_RPAREN);
@@ -436,7 +437,7 @@ Exit_GetInvTemperatures:
 
   if (bErr)
     printf ("Syntax: Inverse temperatures (nInvTemperatures, "
-            "<n inverse temperature values in [0,1] interval>)\n\n");
+            "<n increasing inverse temperature values >= 0>)\n\n");
 
   return (!bErr);
 
@@ -1056,7 +1057,7 @@ static int vrgiGibbsArgTypes[NGIBBS_ARGS] = { LX_STRING, LX_STRING, LX_STRING,
     }
 
     panal->gd.nMaxIter     = atol(vrgszlexArgs[3]);
-    panal->gd.nSimTypeFlag = atol(vrgszlexArgs[4]); /* should be 0 to 4 */
+    panal->gd.nSimTypeFlag = atol(vrgszlexArgs[4]); /* should be 0 to 5 */
     panal->gd.nPrintFreq   = atol(vrgszlexArgs[5]);
     panal->gd.nPrintIter   = atol(vrgszlexArgs[6]);
 
@@ -1064,7 +1065,7 @@ static int vrgiGibbsArgTypes[NGIBBS_ARGS] = { LX_STRING, LX_STRING, LX_STRING,
 
     if (((panal->gd.nSimTypeFlag==1) && (panal->gd.szGrestart == NULL)) || 
         ((panal->gd.nSimTypeFlag==2) && (panal->gd.szGrestart == NULL))) {
-      printf ("Error: if printPredFlag is one or two a restart file must be "
+      printf ("Error: if simTypeFlag is one or two a restart file must be "
               "given - Exiting\n\n");
       exit (0);
     }
@@ -1954,13 +1955,13 @@ void NewExperiment (PINPUTBUF pibIn)
     printf ("Reading experiment %d.\n", panal->expGlobal.iExp);
   }
   else {
-    plevel = panal->pLevels[panal->iInstances-1];
+    plevel = panal->pLevels[panal->iInstances - 1];
     for (n = 0; n < panal->iCurrentDepth-1; ++n) {
-      plevel = plevel->pLevels[plevel->iInstances-1];
+      plevel = plevel->pLevels[plevel->iInstances - 1];
     }
     if (plevel->iInstances == MAX_INSTANCES - 1)
       ReportError (pibIn, RE_TOOMANYINST | RE_FATAL, "NewExperiment", NULL);
-    n = panal->pCurrentLevel[panal->iCurrentDepth-1]->iInstances++;
+    n = panal->pCurrentLevel[panal->iCurrentDepth - 1]->iInstances++;
     if (!(plevel = plevel->pLevels[n] = (PLEVEL)malloc(sizeof(LEVEL))))
       ReportError (pibIn, RE_OUTOFMEM | RE_FATAL, "NewExperiment", NULL);
     plevel->iInstances = 0;
@@ -2061,14 +2062,17 @@ int SetLevel(PINPUTBUF pibIn)
 
   if (panal->iCurrentDepth == 0) {
 
-    if (panal->iInstances == MAX_INSTANCES - 1)
-      ReportError (pibIn, RE_TOOMANYINST | RE_FATAL, "SetLevel", NULL);
-
     plevel = panal->pLevels[panal->iInstances++] 
            = (PLEVEL) malloc (sizeof (LEVEL));
     if (plevel == NULL)
       ReportError (pibIn, RE_OUTOFMEM | RE_FATAL, "SetLevel", NULL);
 
+    /* we do not want more than 1 top level, exit */
+    if (panal->iInstances > 1) {
+      printf("Error: only one top level is allowed - Exiting.\n");
+      exit(0);
+    }
+
     plevel->iSequence = panal->iInstances;
     printf ("New level - depth 1, instance %d\n", panal->iInstances);
 
@@ -2078,7 +2082,7 @@ int SetLevel(PINPUTBUF pibIn)
     plevel = panal->pLevels[panal->iInstances-1];
 
     for (n = 0; n < panal->iCurrentDepth-1; ++n)
-      plevel = plevel->pLevels[plevel->iInstances-1];
+      plevel = plevel->pLevels[plevel->iInstances - 1];
 
     if (plevel->iInstances == MAX_INSTANCES - 1)
       ReportError (pibIn, RE_TOOMANYINST | RE_FATAL, "SetLevel", NULL);
@@ -2092,7 +2096,7 @@ int SetLevel(PINPUTBUF pibIn)
       ReportError (pibIn, RE_OUTOFMEM | RE_FATAL, "SetLevel", NULL);
 
     plevel->iSequence = n + 1;
-    printf("New level - depth %d, instance %d\n", panal->iCurrentDepth+1,
+    printf("New level - depth %d, instance %d\n", panal->iCurrentDepth + 1,
            panal->pCurrentLevel[panal->iCurrentDepth-1]->iInstances);
   }
 
@@ -2131,7 +2135,8 @@ BOOL EndLevel (PANALYSIS panal)
 */
 void FreeLevels (PANALYSIS panal)
 {
-  BYTE n;
+  int n;
+
   for (n = 0; n < panal->iInstances; n++)
     if (panal->pLevels[n] != NULL)
       FreeOneLevel (panal->pLevels[n]);
@@ -2156,7 +2161,7 @@ int FreeMCLists (PVOID pData, PVOID pUserInfo)
 */
 void FreeOneLevel (PLEVEL plevel)
 {
-  BYTE n;
+  int n;
 
   for (n = 0; n < plevel->iInstances; n++)
     if (plevel->pLevels[n] != NULL)
@@ -2175,6 +2180,7 @@ void FreeOneLevel (PLEVEL plevel)
     free (plevel->rgpMCVars);
   if (plevel->nLikes > 0)
     free (plevel->rgpLikes);
+
   free(plevel);
 
 } /* FreeOneLevel */
